package com.pj.task;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.pj.api.client.level_one_server.LevelOneServerInterface;
import com.pj.api.client.level_two_server.LevelTwoServerInterface;
import com.pj.bank_byte.BankByte;
import com.pj.bank_byte.BankByteMapper;
import com.pj.bank_info.BankInfo;
import com.pj.bank_info.BankInfoMapper;
import com.pj.bank_info_charges.BankInfoCharges;
import com.pj.bank_info_charges.BankInfoChargesService;
import com.pj.common.core.utils.StringUtils;
import com.pj.enummj.FinishStatus;
import com.pj.utils.ht.AESUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.transaction.annotation.Transactional;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;
import java.io.IOException;
import java.io.StringReader;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Configuration
@EnableScheduling
@Transactional(rollbackFor = Exception.class)
public class BankByteTask {

    /**
     * 银行回执字节码mapper
     */
    @Autowired
    private BankByteMapper bankByteMapper;
    /**
     * 银行回执信息mapper
     */
    @Autowired
    private BankInfoMapper bankInfoMapper;
    /**
     * 银行回执信息-手续费
     */
    @Autowired
    private BankInfoChargesService bankInfoChargesService;

    @Autowired
    private LevelOneServerInterface levelOneServerInterface;
    @Autowired
    private LevelTwoServerInterface levelTwoServerInterface;

    /**
     * 启动定时任务
     */
    //@Scheduled(cron = "*/10 * * * * ?")  // 每10秒扫描一次
    private void bankByteTask() {

        //扫描表内是否有待处理任务
        List<BankByte> byteList = bankByteMapper.selectList(
                new LambdaQueryWrapper<BankByte>().eq(BankByte::getFinishStatus, FinishStatus.FINISH_STATUS_ZERO.getCode()).orderByAsc(BankByte::getCreateTime));
        //判断是否有任务,没有则返回
        if (byteList.size() == 0) {
            System.err.println("\n  ---  暂无待处理【银行回执信息】。" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + "\n");
            return;
        }
        //执行任务处理流程
        byteList.forEach(item -> {
            String body = item.getBody();
            //数据解析
            String xmlStr = body;//AESUtil.decryptCBC(body, RabbitMQOne.ACC_KEY);

            BankInfo bankInfo = new BankInfo();
            List<BankInfoCharges> chargesList = new ArrayList<>();
            boolean isMKT008 = StringUtils.containsIgnoreCase(xmlStr,"MKT008");
            boolean hasCommission = StringUtils.containsIgnoreCase(xmlStr, "<results xsi:type=\"orderResultDTO\">");

            //银行回执——headers——获取头部字段
            if(isMKT008){//二级市场-订单回执报文-头部处理
                String headersStr = xmlStr.substring(xmlStr.indexOf("<headers>")-9, xmlStr.indexOf("</headers>"+10));
                try {
                    // 创建DOM解析器
                    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                    DocumentBuilder builder = factory.newDocumentBuilder();

                    // 解析XML字符串，获取Document对象
                    Document document = builder.parse(new InputSource(new StringReader(headersStr)));

                    // 创建XPath对象
                    XPath xpath = XPathFactory.newInstance().newXPath();

                    bankInfo.setSn(xpath.evaluate("/headers/sn", document));
                    bankInfo.setDate(xpath.evaluate("/headers/date", document));
                    bankInfo.setOrganization(xpath.evaluate("/headers/organization", document));
                    bankInfo.setBankCode(xpath.evaluate("/headers/bankCode", document));
                    bankInfo.setOperation(xpath.evaluate("/headers/operation", document));

                } catch (ParserConfigurationException e) {
                    e.printStackTrace();
                } catch (SAXException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                } catch (XPathExpressionException e) {
                    e.printStackTrace();
                }
            }
            else {//一级市场-订单回执报文-头部处理
                String headersStr = xmlStr.substring(xmlStr.indexOf("<headers>")+9, xmlStr.indexOf("</headers>"));
                String[] headersSplit = headersStr.split("</item>");
                for (String s : headersSplit) {
                    if(StringUtils.containsIgnoreCase(s,"sn")) {
                        String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                        bankInfo.setSn(substring);
                    }
                    if(StringUtils.containsIgnoreCase(s,"date")) {
                        String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                        bankInfo.setDate(substring);
                    }
                    if(StringUtils.containsIgnoreCase(s,"organization")) {
                        String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                        bankInfo.setOrganization(substring);
                    }
                    if(StringUtils.containsIgnoreCase(s,"bankCode")) {
                        String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                        bankInfo.setBankCode(substring);
                    }
                    if(StringUtils.containsIgnoreCase(s,"operation")) {
                        String substring = s.substring(s.indexOf("<value>")+7, s.indexOf("</value>"));
                        bankInfo.setOperation(substring);
                    }
                }
            }

            //银行回执——payload——获取responseDTO
            String payloadStr = xmlStr.substring(xmlStr.indexOf("</headers>")+10, xmlStr.indexOf("</message>"));
            if(isMKT008 && hasCommission) {//是【MKT008】报文(二级市场订单报文回执)，并且是【有手续费模式】
                payloadStr = xmlStr.substring(xmlStr.indexOf("</headers>")+10, xmlStr.indexOf("<results xsi:type=\"orderResultDTO\">"));
                payloadStr += "</payload>";

                String resultsStr = xmlStr.substring(xmlStr.indexOf("<results xsi:type=\"orderResultDTO\">"), xmlStr.lastIndexOf("</results>"));
                String[] resultsSplit = resultsStr.split("</results>");

                try {
                    // 创建DOM解析器
                    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                    DocumentBuilder builder = factory.newDocumentBuilder();
                    // 创建XPath对象
                    XPath xpath = XPathFactory.newInstance().newXPath();
                    for (String s : resultsSplit) {
                        BankInfoCharges charges = new BankInfoCharges();
                        // 解析XML字符串，获取Document对象
                        Document document = builder.parse(new InputSource(new StringReader(s)));

                        charges.setSn(xpath.evaluate("/results/sn", document));
                        charges.setType(xpath.evaluate("/results/type", document));
                        charges.setCode(xpath.evaluate("/results/code", document));
                        charges.setNote(xpath.evaluate("/results/note", document));
                        charges.setResult(xpath.evaluate("/results/result", document));
                        chargesList.add(charges);
                    }

                } catch (ParserConfigurationException e) {
                    e.printStackTrace();
                } catch (SAXException e) {
                    e.printStackTrace();
                } catch (IOException e) {
                    e.printStackTrace();
                } catch (XPathExpressionException e) {
                    e.printStackTrace();
                }

            }
            try {
                // 创建DOM解析器
                DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                DocumentBuilder builder = factory.newDocumentBuilder();

                // 解析XML字符串，获取Document对象
                Document document = builder.parse(new InputSource(new StringReader(payloadStr)));

                // 创建XPath对象
                XPath xpath = XPathFactory.newInstance().newXPath();

                bankInfo.setCode(xpath.evaluate("/payload/code", document));
                bankInfo.setSource(xpath.evaluate("/payload/source", document));
                bankInfo.setResult(xpath.evaluate("/payload/result", document));// 二级市场订单扣款(有手续费模式)：< result> 可忽略
                bankInfo.setNote(xpath.evaluate("/payload/note", document));
                bankInfo.setType(xpath.evaluate("/payload/type", document));

            } catch (ParserConfigurationException e) {
                e.printStackTrace();
            } catch (SAXException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } catch (XPathExpressionException e) {
                e.printStackTrace();
            }


            //保存银行回执信息
            int insert = bankInfoMapper.insert(bankInfo);
            if(isMKT008 && hasCommission) {
                chargesList.forEach(Charges -> Charges.setBankInfoId(bankInfo.getId()));
                boolean saveBatch = bankInfoChargesService.saveBatch(chargesList);
                System.out.println("保存到银行回执信息-手续费："+saveBatch);
            }

            boolean result = false;
            //（一级市场）订单扣款-成功
            if(bankInfo.getOperation().equalsIgnoreCase("SXB011")) {
                //参数：一级市场边民订单-订单编号
                result = levelOneServerInterface.confirmOrderFromBank(bankInfo.getSource(),1);
            }
            //（一级市场）订单扣款-失败
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB012")) {
                result = levelOneServerInterface.confirmOrderFromBank(bankInfo.getSource(),2);
            }
            //（一级市场）订单退款-成功
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB014")) {
                result = levelOneServerInterface.chargebackFromBank(bankInfo.getSource(),1, bankInfo.getNote());
            }
            //（一级市场）订单退款-失败
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB015")) {
                result = levelOneServerInterface.chargebackFromBank(bankInfo.getSource(),2, bankInfo.getNote());
            }
            //（一级市场-通车批信息）订单打款-成功
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB017")) {
                //参数：一级市场边民订单-订单编号, n个
                result = levelOneServerInterface.orderFinishFromBank(bankInfo.getSource(), 1);
            }
            //（一级市场-通车批信息）订单打款-失败
            else if(bankInfo.getOperation().equalsIgnoreCase("SXB018")) {
                result = levelOneServerInterface.orderFinishFromBank(bankInfo.getSource(), 2);
            }

            //（二级市场）订单扣款            001：扣款订单    002：退款订单    003：打款订单
            else if(bankInfo.getOperation().equalsIgnoreCase("MKT008") && "001".equals(bankInfo.getType())) {
                if(!hasCommission) {//无手续费模式——采购商支付扣款
                    if("200".equals(bankInfo.getCode())) {//回执代码为200则表示支付成功
                        //参数：二级市场边民订单-订单编号
                        result = levelTwoServerInterface.payTax(bankInfo.getSource(),1);
                    } else {//支付失败
                        result = levelTwoServerInterface.payTax(bankInfo.getSource(),2);
                    }
                }
                else {//有手续费模式——边民缴税以及各服务费
                    if("200".equals(bankInfo.getCode())) {//回执代码为200则表示缴费成功
                        //参数：二级市场边民订单-订单编号
                        result = levelTwoServerInterface.payTaxFromBank(bankInfo.getSource(),1);
                    } else {//缴费失败
                        result = levelTwoServerInterface.payTaxFromBank(bankInfo.getSource(),2);
                    }
                }
            }
            //（二级市场）订单退款-----暂无调用订单退款
            //else if(bankInfo.getOperation().equalsIgnoreCase("MKT008") && "002".equals(bankInfo.getType())) { }

            //数据处理完成状态
            if(result) {//处理成功
                item.setFinishStatus(FinishStatus.FINISH_STATUS_ONE.getCode());
            } else {//处理失败
                item.setFinishStatus(FinishStatus.FINISH_STATUS_TWO.getCode());
            }
            bankByteMapper.updateById(item);

        });

    }

}
