package com.pj.task;

import com.alibaba.fastjson.JSON;
import com.pj.api.dto.PaymentDto;
import com.pj.api.dto.PaymentTwoDto;
import com.pj.dto.DataDto;
import com.pj.rabbitmq.MessageSender;

import com.pj.xml.MessageXML;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class PaymentService {

    @Autowired
    private MessageSender messageSender;

    public void testSend(DataDto dataDto) {
        System.out.println("开始发送----------\n"+dataDto.toString());
        messageSender.sendToLevelOneMQ(JSON.toJSONString(dataDto));
        System.out.println("结束发送----------");
    }

    /**
     * ※ 订单扣款
     * 报文名称：SXB010.xml  数据流向：一级市场--->银行
     * @param paymentDto 一级市场订单信息
     */
    public void sendSXB010(PaymentDto paymentDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentDto));
        String xml = MessageXML.SXB010(paymentDto);
        messageSender.sendToLevelOneMQ(xml);
        //mq发送信息
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }

    /**
     * ※ 订单退款
     * 报文名称：SXB013.xml  数据流向：一级市场--->银行
     * @param paymentDto 一级市场订单信息
     */
    public void sendSXB013(PaymentDto paymentDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentDto));
        String xml = MessageXML.SXB013(paymentDto);
        messageSender.sendToLevelOneMQ(xml);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }

    /**
     * ※ 通关车批信息同步(车出一级市场时)
     * 报文名称：SXB023.xml  数据流向：一级市场--->银行
     * @param paymentDto 一级市场订单信息
     */
    public void sendSXB023(PaymentDto paymentDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentDto));
        String xml = MessageXML.SXB023(paymentDto);
        messageSender.sendToLevelOneMQ(xml);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }

    /**
     * ※ 订单扣款（无手续费）
     * 报文名称：SXB023.xml  数据流向：二级市场--->银行
     * @param paymentTwoDto 订单信息
     */
    public void sendMKT006_NoCommission(PaymentTwoDto paymentTwoDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentTwoDto));
        String xml = MessageXML.MKT006_NoCommission(paymentTwoDto);
        messageSender.sendToLevelTwoMQ(xml);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }

    /**
     * ※ 订单扣款（+手续费）
     * 报文名称：SXB023.xml  数据流向：二级市场--->银行
     * @param paymentTwoDto 订单信息
     */
    public void sendMKT006(PaymentTwoDto paymentTwoDto) {
        log.info("send data to ht:{}", JSON.toJSONString(paymentTwoDto));
        String xml = MessageXML.MKT006(paymentTwoDto);
        messageSender.sendToLevelTwoMQ(xml);
        log.info("send mq msg finish:{}", JSON.toJSONString(xml));
    }


}
